@@ -1,8 +1,8 @@
-/* aide, Advanced Intrusion Detection Environment
+/*
+ * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2002,2004-2006,2010,2011 Rami Lehti, Pablo
- * Virolainen, Richard van den Berg, Hannes von Haugwitz
- * $Header$
+ * Copyright (C) 1999-2002, 2004-2006, 2010-2011, 2019 Rami Lehti,
+ *               Pablo Virolainen, Richard van den Berg, Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -14,10 +14,11 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
+
 #ifndef _AIDE_H_INCLUDED
 #define _AIDE_H_INCLUDED
 
@@ -32,7 +33,7 @@
 #if HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
-#include "report.h"
+#include "error.h"
 
 #ifndef TEMP_FAILURE_RETRY
 /* Evaluate EXPRESSION, and repeat as long as it returns -1 with errno'
@@ -46,16 +47,6 @@
         __result; }))
 #endif
 
-#if !defined HAVE_VSNPRINTF || !defined HAVE_C99_VSNPRINTF
-#define vsnprintf rsync_vsnprintf
-int vsnprintf(char *str, size_t count, const char *fmt, va_list args);
-#endif
-
-#if !defined HAVE_SNPRINTF || !defined HAVE_C99_VSNPRINTF
-#define snprintf rsync_snprintf
-int snprintf(char *str,size_t count,const char *fmt,...);
-#endif
-
 #ifndef O_NOATIME
 #if defined(__linux__) && (defined(__i386__) || defined(__PPC__))
 #define O_NOATIME 01000000
@@ -68,20 +59,6 @@ int snprintf(char *str,size_t count,const char *fmt,...);
 # define HAVE_STRTOIMAX
 #endif
 
-#if defined HAVE_OFF64_TYPE && SIZEOF_OFF64_T == SIZEOF_LONG_LONG || !defined HAVE_OFF64_TYPE && SIZEOF_OFF_T == SIZEOF_LONG_LONG
-# ifdef HAVE_STRTOLL
-#  define AIDE_STRTOLL_FUNC strtoll
-# else
-#  ifdef HAVE_STRTOIMAX
-#   define AIDE_STRTOLL_FUNC strtoimax
-#  else
-#   define AIDE_STRTOLL_FUNC strtol
-#  endif
-# endif
-#else
-# define AIDE_STRTOLL_FUNC strtol
-#endif
-
 #ifndef __NetBSD__
 #ifndef _POSIX_C_SOURCE
 /* For _POSIX_THREAD_SEMANTICS _REENTRANT */
