@@ -1,9 +1,9 @@
-/* aide, Advanced Intrusion Detection Environment
- * vi: ts=2 sw=2
+/*
+ * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2006,2010,2011,2016 Rami Lehti, Pablo Virolainen,
- * Richard van den Berg, Mike Markley, Hannes von Haugwitz
- * $Header$
+ * Copyright (C) 1999-2006, 2010-2011, 2016-2017, 2019-2021 Rami Lehti,
+ *               Pablo Virolainen, Mike Markley, Richard van den Berg,
+ *               Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -15,9 +15,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "aide.h"
@@ -55,16 +55,12 @@
 #endif
 
 static DIR *dirh = NULL;
-static struct AIDE_DIRENT_TYPE *entp = NULL;
-static struct AIDE_DIRENT_TYPE **resp = NULL;
+static struct dirent *entp = NULL;
 
 static struct seltree *r = NULL;
 
 
-#if defined HAVE_READDIR && !defined HAVE_READDIR_R
 static long td = -1;
-#endif
-static int rdres = 0;
 
 static int root_handled = 0;
 
@@ -79,32 +75,19 @@ static DIR *open_dir(char* path) {
 
 static void next_in_dir (void)
 {
-#ifdef HAVE_READDIR_R
-	if (dirh != NULL)
-		rdres = AIDE_READDIR_R_FUNC (dirh, entp, resp);
-#else
-#ifdef HAVE_READDIR
 	if (dirh != NULL) {
-		entp = AIDE_READDIR_FUNC (dirh);
+		entp = readdir (dirh);
 		if(entp!=NULL)
 			td = telldir(dirh);
 		else
 			td=-1;
 	}
-#endif
-#endif
 
 }
 
 static int in_this (void)
 {
-#ifdef HAVE_READDIR_R
-	return (dirh != NULL && rdres == 0 && (*resp) != NULL);
-#else
-#ifdef HAVE_READDIR
 	return (dirh != NULL && entp != NULL && td >= 0);
-#endif
-#endif
 }
 
 static char *name_construct (const char *s)
@@ -117,12 +100,12 @@ static char *name_construct (const char *s)
 		len++;
 	}
 
-	ret = (char *) malloc (len);
+	ret = (char *) checked_malloc (len);
 	ret[0] = (char) 0;
 	strncpy(ret, conf->root_prefix, conf->root_prefix_length+1);
 	strncat (ret, r->path, len2);
 	if (r->path[len2 - 1] != '/') {
-		strncat (ret, "/", 1);
+		strcat (ret, "/");
 	}
 	strcat (ret, s);
 	return ret;
@@ -133,8 +116,6 @@ void add_child (db_line * fil)
 	int i;
 	struct seltree *new_r;
 
-	error (255, "Adding child %s\n", fil->filename);
-
 	new_r = get_seltree_node (r, fil->filename);
 	if (new_r != NULL) {
 		if (S_ISDIR (fil->perm_o)) {
@@ -146,12 +127,13 @@ void add_child (db_line * fil)
 		return;
 	}
 
-	new_r = malloc (sizeof (seltree));
+	log_msg(LOG_LEVEL_DEBUG, "add child '%s' to %s", fil->filename, r->path);
+
+	new_r = checked_malloc (sizeof (seltree));
 
-	new_r->attr = 0;
 	i = strlen (fil->filename);
 
-	new_r->path = malloc (i + 1);
+	new_r->path = checked_malloc (i + 1);
 	strncpy(new_r->path, fil->filename, i+1);
 	new_r->childs = NULL;
 	new_r->sel_rx_lst = NULL;
@@ -159,6 +141,7 @@ void add_child (db_line * fil)
 	new_r->equ_rx_lst = NULL;
 	new_r->parent = r;
 	new_r->checked = 0;
+	new_r->changed_attrs=0;
 	new_r->new_data = NULL;
 	new_r->old_data = NULL;
 	if (S_ISDIR (fil->perm_o)) {
@@ -170,15 +153,15 @@ void add_child (db_line * fil)
 	r->childs = list_sorted_insert (r->childs, new_r, compare_node_by_path);
 }
 
-static int get_file_status(char *filename, struct AIDE_STAT_TYPE *fs) {
+static int get_file_status(char *filename, struct stat *fs) {
     int sres = 0;
-    sres = AIDE_LSTAT_FUNC(filename,fs);
+    sres = lstat(filename,fs);
     if(sres == -1){
         char* er = strerror(errno);
         if (er == NULL) {
-            error(0,"get_file_status: lstat() failed for %s. strerror() failed for %i\n", filename, errno);
+            log_msg(LOG_LEVEL_WARNING, "get_file_status: lstat() failed for %s. strerror() failed with %i", filename, errno);
         } else {
-            error(0,"get_file_status: lstat() failed for %s: %s\n", filename, er);
+            log_msg(LOG_LEVEL_WARNING, "get_file_status: lstat() failed for %s: %s", filename, er);
         }
     }
     return sres;
@@ -189,32 +172,28 @@ static int get_file_status(char *filename, struct AIDE_STAT_TYPE *fs) {
   Now implemented with goto-statement. Yeah, it's ugly and easy.
 */
 
-db_line *db_readline_disk ()
+db_line *db_readline_disk (bool dry_run)
 {
 	db_line *fil = NULL;
-	DB_ATTR_TYPE attr;
+	rx_rule *rule = NULL;
 	char *fullname;
 	int add = 0;
-	struct AIDE_STAT_TYPE fs;
+	struct stat fs;
 
 	/* root needs special handling */
 	if (!root_handled) {
 		root_handled = 1;
-		fullname=malloc((conf->root_prefix_length+2)*sizeof(char));
+		fullname=checked_malloc((conf->root_prefix_length+2)*sizeof(char));
 		strncpy(fullname, conf->root_prefix, conf->root_prefix_length+1);
-		strncat (fullname, "/", 1);
+		strcat (fullname, "/");
 		if (!get_file_status(fullname, &fs)) {
-		add = check_rxtree (&fullname[conf->root_prefix_length], conf->tree, &attr, fs.st_mode);
-		error (240, "%s match=%d, tree=%p, attr=%llu\n", &fullname[conf->root_prefix_length], add,
-					 conf->tree, attr);
+		add = check_rxtree (&fullname[conf->root_prefix_length], conf->tree, &rule, get_restriction_from_perm(fs.st_mode), dry_run);
 
 		if (add > 0) {
-			fil = get_file_attrs (fullname, attr, &fs);
-
-			error (240, "%s attr=%llu\n", &fullname[conf->root_prefix_length], attr);
+            log_msg(LOG_LEVEL_DEBUG, "get file atttributes '%s'", &fullname[conf->root_prefix_length]);
+			fil = get_file_attrs (fullname, rule->attr, &fs, dry_run);
 
 			if (fil != NULL) {
-				error (240, "%s attr=%llu\n", fil->filename, fil->attr);
 				return fil;
 			}
         }
@@ -253,18 +232,13 @@ recursion:
 		    free (fullname);
 		    goto recursion;
 		}
-		add = check_rxtree (&fullname[conf->root_prefix_length], conf->tree, &attr, fs.st_mode);
-		error (240, "%s match=%d, tree=%p, attr=%llu\n", &fullname[conf->root_prefix_length], add,
-					 conf->tree, attr);
+		add = check_rxtree (&fullname[conf->root_prefix_length], conf->tree, &rule, get_restriction_from_perm(fs.st_mode), dry_run);
 
 		if (add > 0) {
-			fil = get_file_attrs (fullname, attr, &fs);
+            log_msg(LOG_LEVEL_DEBUG, "get file atttributes '%s'", &fullname[conf->root_prefix_length]);
+			fil = get_file_attrs (fullname, rule->attr, &fs, dry_run);
 
-			error (240, "%s attr=%llu\n", &fullname[conf->root_prefix_length], attr);
-
-			if (fil != NULL) {
-				error (240, "%s attr=%llu\n", fil->filename, fil->attr);
-			} else {
+			if (fil == NULL) {
 				/*
 				   Something went wrong during read process -> 
 				   Let's try next one.
@@ -273,7 +247,7 @@ recursion:
 				goto recursion;					// return db_readline_disk(db);
 			}
 
-			if (add == 1) {
+			if (add == SELECTIVE_MATCH) {
 				/*
 				   add_children -> if dir, then add to children list.
 				 */
@@ -282,7 +256,7 @@ recursion:
 				   accordingly
 				 */
 				add_child (fil);
-			} else if (add == 2) {
+			} else if (add == EQUAL_MATCH) {
 				/*
 				   Don't add to children list.
 				 */
@@ -313,7 +287,7 @@ recursion:
 			return NULL;
 		}
 
-		error (255, "r->childs %p, r->parent %p, r->checked %i\n", r->childs,
+		log_msg(LOG_LEVEL_TRACE, "r->childs %p, r->parent %p, r->checked %i", r->childs,
 					 r->parent, r->checked);
 
 		if ((0 == (r->checked & NODE_CHECKED)) && r->childs != NULL) {
@@ -332,16 +306,16 @@ recursion:
 
 				rr = (seltree *) l->data;
 
-				error (255, "rr->checked %i\n", rr->checked);
+				log_msg(LOG_LEVEL_TRACE, "rr->checked %i", rr->checked);
 				rr->checked |= NODE_TRAVERSE;
 
 				r = rr;
 
-				error (255, "r->childs %p, r->parent %p,r->checked %i\n",
+				log_msg (LOG_LEVEL_TRACE, "r->childs %p, r->parent %p,r->checked %i",
 							 r->childs, r->parent, r->checked);
-				fullname=malloc((conf->root_prefix_length+strlen(r->path)+1)*sizeof(char));
+				fullname=checked_malloc((conf->root_prefix_length+strlen(r->path)+1)*sizeof(char));
 				strncpy(fullname, conf->root_prefix, conf->root_prefix_length+1);
-				strncat(fullname, r->path, strlen(r->path));
+				strcat(fullname, r->path);
 				dirh=open_dir(fullname);
 				if (! dirh) {
 
@@ -366,20 +340,20 @@ recursion:
 						   The only way a nonexistent dirnode can have children is by 
 						   having rules referring to them.
 						 */
-						error (10,
-									 "There are rules referring to non-existent directory %s\n", fullname);
+						log_msg (LOG_LEVEL_RULE,
+									 "There are rules referring to non-existent directory %s", fullname);
 					} else if (errno != ENOTDIR) {
 						/* We print the message unless it is "Not a directory". */
 						char *er = strerror (errno);
 						if (er != NULL) {
-							error (3, "open_dir(): %s: %s\n", er, fullname);
+							log_msg(LOG_LEVEL_WARNING, "open_dir() failed for %s: %s", fullname, er);
 						} else {
-							error (3, "open_dir(): %i: %s\n", errno, fullname);
+							log_msg(LOG_LEVEL_WARNING, "open_dir(): failed for %s: %i", fullname, errno);
 						}
 					}
 					r->checked |= NODE_TRAVERSE | NODE_CHECKED;
 					r = r->parent;
-					error (255, "dropping back to parent\n");
+					log_msg(LOG_LEVEL_TRACE, "dropping back to parent");
 				}
 				free(fullname);
 			} else {
@@ -389,7 +363,7 @@ recursion:
 				if (r == NULL) {
 					return NULL;
 				}
-				error (255, "dropping back to parent\n");
+				log_msg(LOG_LEVEL_TRACE, "dropping back to parent");
 			}
 			goto recursion;
 		}
@@ -417,26 +391,9 @@ int db_disk_init ()
 
 	r = conf->tree;
 
-#  ifdef HAVE_READDIR_R
-	resp = (struct AIDE_DIRENT_TYPE **)
-		malloc (sizeof (struct AIDE_DIRENT_TYPE) + _POSIX_PATH_MAX);
-	entp = (struct AIDE_DIRENT_TYPE *)
-		malloc (sizeof (struct AIDE_DIRENT_TYPE) + _POSIX_PATH_MAX);
-#  else
-#   ifdef HAVE_READDIR
-	/*
-	   Should we do something here?
-
-	 */
-#   else
-#    error AIDE needs readdir or readdir_r
-#   endif
-#  endif
-
-
-	char* fullname=malloc((conf->root_prefix_length+2)*sizeof(char));
+	char* fullname=checked_malloc((conf->root_prefix_length+2)*sizeof(char));
 	strncpy(fullname, conf->root_prefix, conf->root_prefix_length+1);
-	strncat (fullname, "/", 1);
+	strcat (fullname, "/");
 	dirh=open_dir(fullname);
 	free(fullname);
 
@@ -448,6 +405,3 @@ int db_disk_init ()
   backup/restore software. Hence the functions db_writespec_disk,
   db_writeline_disk and db_close_disk aren't declared.
  */
-
-const char *aide_key_6 = CONFHMACKEY_06;
-const char *db_key_6 = DBHMACKEY_06;
